; ataio.h
;
; header file for ataio.asm
;
; Copyright (C) 2000, Suzhe. See file COPYING and CREDITS for details.
;
; This file is based on the Public Domain software -- 
; ATA LOW LEVEL I/O DRIVER by Hale Landis (hlandis@ibm.net)
;

%define ATA_DRIVER_VERSION "13J"

; Global defines -- ATA register and register bits.
;
; command block & control block regs
;
; these are the offsets into pio_reg_addrs[]

%define CB_DATA  0   ; data reg         in/out pio_base_addr1+0
%define CB_ERR   1   ; error            in     pio_base_addr1+1
%define CB_FR    1   ; feature reg         out pio_base_addr1+1
%define CB_SC    2   ; sector count     in/out pio_base_addr1+2
%define CB_SN    3   ; sector number    in/out pio_base_addr1+3
%define CB_CL    4   ; cylinder low     in/out pio_base_addr1+4
%define CB_CH    5   ; cylinder high    in/out pio_base_addr1+5
%define CB_DH    6   ; device head      in/out pio_base_addr1+6
%define CB_STAT  7   ; primary status   in     pio_base_addr1+7
%define CB_CMD   7   ; command             out pio_base_addr1+7
%define CB_ASTAT 8   ; alternate status in     pio_base_addr2+6
%define CB_DC    8   ; device control      out pio_base_addr2+6
%define CB_DA    9   ; device address   in     pio_base_addr2+7

; error reg (CB_ERR) bits

%define CB_ER_ICRC 0x80    ; ATA Ultra DMA bad CRC
%define CB_ER_BBK  0x80    ; ATA bad block
%define CB_ER_UNC  0x40    ; ATA uncorrected error
%define CB_ER_MC   0x20    ; ATA media change
%define CB_ER_IDNF 0x10    ; ATA id not found
%define CB_ER_MCR  0x08    ; ATA media change request
%define CB_ER_ABRT 0x04    ; ATA command aborted
%define CB_ER_NTK0 0x02    ; ATA track 0 not found
%define CB_ER_NDAM 0x01    ; ATA address mark not found

%define CB_ER_P_SNSKEY 0xf0   ; ATAPI sense key (mask)
%define CB_ER_P_MCR    0x08   ; ATAPI Media Change Request
%define CB_ER_P_ABRT   0x04   ; ATAPI command abort
%define CB_ER_P_EOM    0x02   ; ATAPI End of Media
%define CB_ER_P_ILI    0x01   ; ATAPI Illegal Length Indication

; ATAPI Interrupt Reason bits in the Sector Count reg (CB_SC)

%define CB_SC_P_TAG    0xf8   ; ATAPI tag (mask)
%define CB_SC_P_REL    0x04   ; ATAPI release
%define CB_SC_P_IO     0x02   ; ATAPI I/O
%define CB_SC_P_CD     0x01   ; ATAPI C/D

; bits 7-4 of the device/head (CB_DH) reg

%define CB_DH_DEV0 0xa0    ; select device 0
%define CB_DH_DEV1 0xb0    ; select device 1

; status reg (CB_STAT and CB_ASTAT) bits

%define CB_STAT_BSY  0x80  ; busy
%define CB_STAT_RDY  0x40  ; ready
%define CB_STAT_DF   0x20  ; device fault
%define CB_STAT_WFT  0x20  ; write fault (old name)
%define CB_STAT_SKC  0x10  ; seek complete
%define CB_STAT_SERV 0x10  ; service
%define CB_STAT_DRQ  0x08  ; data request
%define CB_STAT_CORR 0x04  ; corrected
%define CB_STAT_IDX  0x02  ; index
%define CB_STAT_ERR  0x01  ; error

; digital output reg (CB_DC) bits

%define CB_DC_HD15   0x08  ; bit should always be set to one
%define CB_DC_SRST   0x04  ; soft reset
%define CB_DC_NIEN   0x02  ; disable interrupts

;**************************************************************

; Most mandtory and optional ATA commands (from ATA-3),

%define CMD_CFA_ERASE_SECTORS            0xC0
%define CMD_CFA_REQUEST_EXT_ERR_CODE     0x03
%define CMD_CFA_TRANSLATE_SECTOR         0x87
%define CMD_CFA_WRITE_MULTIPLE_WO_ERASE  0xCD
%define CMD_CFA_WRITE_SECTORS_WO_ERASE   0x38
%define CMD_CHECK_POWER_MODE1            0xE5
%define CMD_CHECK_POWER_MODE2            0x98
%define CMD_DEVICE_RESET                 0x08
%define CMD_EXECUTE_DEVICE_DIAGNOSTIC    0x90
%define CMD_FLUSH_CACHE                  0xE7
%define CMD_FORMAT_TRACK                 0x50
%define CMD_IDENTIFY_DEVICE              0xEC
%define CMD_IDENTIFY_DEVICE_PACKET       0xA1
%define CMD_IDENTIFY_PACKET_DEVICE       0xA1
%define CMD_IDLE1                        0xE3
%define CMD_IDLE2                        0x97
%define CMD_IDLE_IMMEDIATE1              0xE1
%define CMD_IDLE_IMMEDIATE2              0x95
%define CMD_INITIALIZE_DRIVE_PARAMETERS  0x91
%define CMD_INITIALIZE_DEVICE_PARAMETERS 0x91
%define CMD_NOP                          0x00
%define CMD_PACKET                       0xA0
%define CMD_READ_BUFFER                  0xE4
%define CMD_READ_DMA                     0xC8
%define CMD_READ_DMA_QUEUED              0xC7
%define CMD_READ_MULTIPLE                0xC4
%define CMD_READ_SECTORS                 0x20
%define CMD_READ_VERIFY_SECTORS          0x40
%define CMD_RECALIBRATE                  0x10
%define CMD_SEEK                         0x70
%define CMD_SET_FEATURES                 0xEF
%define CMD_SET_MULTIPLE_MODE            0xC6
%define CMD_SLEEP1                       0xE6
%define CMD_SLEEP2                       0x99
%define CMD_STANDBY1                     0xE2
%define CMD_STANDBY2                     0x96
%define CMD_STANDBY_IMMEDIATE1           0xE0
%define CMD_STANDBY_IMMEDIATE2           0x94
%define CMD_WRITE_BUFFER                 0xE8
%define CMD_WRITE_DMA                    0xCA
%define CMD_WRITE_DMA_QUEUED             0xCC
%define CMD_WRITE_MULTIPLE               0xC5
%define CMD_WRITE_SECTORS                0x30
%define CMD_WRITE_VERIFY                 0x3C

; Public data in ATAIOREG.C

; Extended error information returned by
; reg_reset(), reg_non_data(), reg_pio_data_in() and reg_pio_data_out().

%define FAILBIT15 0x8000   ; extra interrupts detected
%define FAILBIT14 0x4000
%define FAILBIT13 0x2000
%define FAILBIT12 0x1000
%define FAILBIT11 0x0800
%define FAILBIT10 0x0400
%define FAILBIT9  0x0200
%define FAILBIT8  0x0100   ; SC( CD/IO bits) wrong at end of cmd
%define FAILBIT7  0x0080   ; byte count odd at data packet xfer time
%define FAILBIT6  0x0040   ; byte count wrong at data packet xfer time
%define FAILBIT5  0x0020   ; SC (IO bit) wrong at data packet xfer time
%define FAILBIT4  0x0010   ; SC (CD bit) wrong at data packet xfer time
%define FAILBIT3  0x0008   ; byte count wrong at cmd packet xfer time
%define FAILBIT2  0x0004   ; SC wrong at cmd packet xfer time
%define FAILBIT1  0x0002   ; got interrupt before cmd packet xfer
%define FAILBIT0  0x0001   ; slow setting BSY=1 or DRQ=1 after AO cmd
 
struc REG_CMD_INFO
   .flg      resb 1         ; see TRC_FLAG_xxx below
   .ct       resb 1         ; see TRC_TYPE_xxx below
   .cmd      resb 1         ; command code
   .fr1      resb 1         ; feature reg before
   .sc1      resb 1         ; sec cnt before
   .sn1      resb 1         ; sec num before
   .cl1      resb 1         ; cyl low before
   .ch1      resb 1         ; cyl high before
   .dh1      resb 1         ; device head before
   .dc1      resb 1         ; device control before
   .ec       resb 1         ; detailed error code
   .to       resb 1         ; not zero if time out error
   .st2      resb 1         ; status reg after
   .as2      resb 1         ; alt status reg after
   .er2      resb 1         ; error reg after
   .sc2      resb 1         ; sec cnt after
   .sn2      resb 1         ; sec num after
   .cl2      resb 1         ; cyl low after
   .ch2      resb 1         ; cyl high after
   .dh2      resb 1         ; device head after
   .totalBytesXfer  resd 1  ; total bytes transfered
   .failbits        resw 1  ; failure bits (ATAPI protocol errors
   .drqPackets      resd 1  ; number of PIO DRQ packets
endstruc

%define REG_CONFIG_TYPE_NONE  0
%define REG_CONFIG_TYPE_UNKN  1
%define REG_CONFIG_TYPE_ATA   2
%define REG_CONFIG_TYPE_ATAPI 3

; end ataio.h
